//	Author:		Dario Accornero
//	BeDev ID:	10183
//	Mailto:		adario@cs.bu.edu
//	Project:	BeScript
//	Module:
//	The BeScriptPeek application.

/*
	Copyright 2000, Dario Accornero.   All Rights Reserved.
	This file may be used under the terms of the Modified Be Sample Code License
	(the "Software License" document accompanying this file).
*/

//	System headers.

#ifndef _STDIO_H_
#include <stdio.h>
#endif
#ifndef _STDLIB_H_
#include <stdlib.h>
#endif
#ifndef	_STRING_H_
#include <string.h>
#endif
#ifndef	_ALERT_H
#include <Alert.h>
#endif

//	Local headers.

#include "ScriptingElement.h"
#include "ScriptingTargets.h"
#include "ScriptingUtils.h"
#include "BeScriptPeekApp.h"
#include "BeScriptPeekWin.h"
using namespace BeScriptPeek;

//	---	BeScriptPeekApp:
//		Constructor.

BeScriptPeekApp::BeScriptPeekApp(const char* signature)
: BApplication(signature)
{
	fArgumentsReceived = false;
}

//	--- BeScriptPeekApp:
//		Analyze target applications/teams.

void BeScriptPeekApp::ArgvReceived(int32 argc, char** argv)
{
	//	If there is at least one argument on the command line, display structure
	//	of all specified applications.

	CreateAppWindow();
	int	index = 1;
	while (index < argc)
	{
		ScriptingApplication* app = CreateScriptingApplication(argv[index++]);
		if (app)
		{
			//	If everything goes well, only the application object will be displayed --
			//	not its children. This is because we analyze the application in a separate thread
			//	therefore the following method returns immediately.
			
			app->AnalyzeApplication();
			app->PrintToStream();
			fArgumentsReceived = true;	
		}
	}		
}

//	--- BeScriptPeekApp:
//		Possible analyze all running applications if no parameters
//		were given to BeScript.

void BeScriptPeekApp::ReadyToRun()
{
	if (!fArgumentsReceived)
	{
		//	If there are no arguments on the command line, just display a list of all
		//	running applications and their supported suites (if any).

		CreateAppWindow();
		BList* appList = GetRunningApplications(BESCRIPTPEEK_APP_SIGNATURE);
		for (int32 index = 0; index < appList->CountItems(); ++index)
		{
			ScriptingApplication* app = (ScriptingApplication*)appList->ItemAt(index);
			if (app)
			{
				//	If everything goes well, only the application object will be displayed --
				//	not its children. This is because we analyze the application in a separate thread
				//	therefore the following method returns immediately.
			
				app->AnalyzeApplication();
				app->PrintToStream();
			}
		}
		
		//	Get rid of list. This will not actually free the memory pointed to by our elements,
		//	so we're safe to go on.
		
		appList->MakeEmpty();
		delete appList;
	}	
	
	//	We watch the system roster for application starts and terminations.
	
	be_roster->StartWatching(be_app_messenger);
}

//	--- BeScriptPeekApp:
//		Display simple about box.

void BeScriptPeekApp::AboutRequested()
{
	const char* aboutMessage = 	"BeScriptPeek\n"
								"Version 1.0a6\n"
								"©2000 Dario Accornero\n"
								"mailto:bexoft@bexoft.com\n\n"
								
								"A component of BeScript\n\n"
								
								"BMessage and BPropertyInfo (de)hydration by\n"
								"Attila Mezei <attila.mezei@mail.datanet.hu>\n\n"
								
								"PrefilledBitmap and BetterScrollView by\n"
								"Brian Tietz <briant@timelinevista.com>\n\n"
								
								"Dynamic specifiers are shown in red\n"
								"Option-click to expand/collapse entire hierarchies\n"
								"Double-click to rescan applications or to retrieve\n"
								"the current selection dynamic name otherwise\n"
								;
	BAlert* aboutAlert = new BAlert("", aboutMessage, "Yeah");
	aboutAlert->Go(0);
}

//	--- BeScriptPeekApp:
//		Simply pass the "add application" message on to the main window.

void BeScriptPeekApp::MessageReceived(BMessage* msg)
{
	switch (msg->what)
	{
		case BESCRIPTPEEK_ADD_APPLICATION:
			fWindow->PostMessage(msg);
		break;
		case B_SOME_APP_LAUNCHED:
		{
			const char* signature;
			if (msg->FindString("be:signature", &signature) == B_OK)
			{
				if (fWindow->WatchLaunch())
				{
					ScriptingApplication* app = CreateScriptingApplication(signature);
					if (app)
						app->AnalyzeApplication();
				}
			}
		}
		break;
		case B_SOME_APP_QUIT:
		{
			const char* signature;
			if (msg->FindString("be:signature", &signature) == B_OK)
			{
				if (fWindow->WatchQuit())
				{
					bool found = false;
					ScriptingElement* element = 0;
					if (fWindow->Lock())
					{
						DOutlineListView* listView = fWindow->ObjectsList();
						int32 items = listView->FullListCountItems();
						for (int32 item = 0; !found && item < items; ++item)
						{
							element = (ScriptingElement*)listView->ItemAt(item);
							if (element && element->GetInternalName() == signature)
								found = true;
						}
						fWindow->Unlock();
					}
					if (found && element)
					{
						BMessage removeMsg(BESCRIPTPEEK_REMOVE_APPLICATION);
						removeMsg.AddPointer(SF_RESULT, element);
						fWindow->PostMessage(&removeMsg);
					}
				}
			}
		}
		break;
		default:
			BApplication::MessageReceived(msg);
	}
}

//	--- BeScriptPeekApp:
//		Stop watching before we quit.

bool BeScriptPeekApp::QuitRequested()
{
	be_roster->StopWatching(be_app_messenger);
	return BApplication::QuitRequested();
}

//	--- BeScriptPeekApp:
//		Instantiate main window.

void BeScriptPeekApp::CreateAppWindow()
{
	//	By default we now turn off dynamic names.
	
	ScriptingElement::DynamicNames = false;
	BRect frame(50, 50, 50 + defaultWindowWidth, 50 + defaultWindowHeight);
	fWindow = new BeScriptPeekWindow(frame);
	fWindow->Show();
}
