//	Author:		Dario Accornero
//	BeDev ID:	10183
//	Mailto:		adario@cs.bu.edu
//	Project:	BeScript
//	Module:
//	Scripting targets.

/*
	Copyright 2000, Dario Accornero.   All Rights Reserved.
	This file may be used under the terms of the Modified Be Sample Code License
	(the "Software License" document accompanying this file).
*/

#ifndef	SCRIPTINGTARGETS_H
#define	SCRIPTINGTARGETS_H

//	System headers.

#ifndef _ROSTER_H
#include <Roster.h>
#endif

//	Local headers.

#ifndef	SCRIPTINGELEMENT_H
#include "ScriptingElement.h"
#endif

//	CLASSES:
//	A whole application. It contains all scriptable windows (if any),
//	and a complete hierarchy of all the views in the windows.
//	It retains ownership of the messenger.

class _EXPORT ScriptingApplication : public ScriptingElement
{
public:
	ScriptingApplication(	const BString& theName,
							const app_info& theAppInfo,
							BMessenger* theMessenger = 0,
							ScriptingElement* theParent = 0);
	virtual ~ScriptingApplication();

	//	Display element on the console.
	
	virtual	void		PrintToStream() const;

	//	Analyze entire application.
	//	Note that the overloaded method should *not* be called for this kind of object:
	//	call AnalyzeApplication() instead -- it will run the analysis in a fresh thread.
	
	virtual	int32		Analyze(int32 theLevel = 0);
	int32				AnalyzeApplication();
	
	//	Dynamic support.
	
	virtual status_t	FindDynamicName();
	
	//	Interface.
	#ifndef BESCRIPT
	virtual void UpdateItem();
	virtual const BBitmap* GetBitmap() const;
	#endif
	
	//	Rescan application i.e. when a window is shown or hidden.
	
	void Rescan(BView* view = 0);
	
	//	Accessors:
	//	Retrieving information.
	
	inline int32 GetWindows() const			{ return fWindows; }

private:									//	Members.

	app_info	fAppInfo;					//	Application runtime information.
	int32		fWindows;					//	Number of available (including hidden) windows.

	static int32 AnalyzeThread(ScriptingApplication* app);
};

//	A window from an application. It is contained in its parent object (an application).
//	It retains ownership of the messenger.

class _EXPORT ScriptingWindow : public ScriptingElement
{
public:
	//	Flavourful constructors.
	
	ScriptingWindow(	const int32 theNumber = 0,
						BMessenger* theMessenger = 0,
						ScriptingElement* theParent = 0,
						const int32 theLevel = 0,
						bool addSpecifier = true);
	ScriptingWindow(	const char* theName = "",
						BMessenger* theMessenger = 0,
						ScriptingElement* theParent = 0,
						const int32 theLevel = 0,
						bool addSpecifier = true);
	virtual	~ScriptingWindow();

	//	Display element on the console.
	
	virtual	void PrintToStream() const;

	//	Analyze window and its views.
	
	virtual	int32 Analyze(int32 theLevel = 0);
	
	//	Dynamic support.
	
	virtual status_t	FindDynamicName();

	//	Interface.
	#ifndef BESCRIPT
	virtual void UpdateItem();
	virtual const BBitmap* GetBitmap() const;
	#endif

private:	
	void InitObject(const int32 theNumber, bool addSpecifier = true);
	void InitObject(const char* theName, bool addSpecifier = true);
	void InitObject();
};

//	A view from an application window. It is contained either directly
//	in its parent window, or in a parent view.
//	The messenger is shared with the parent object (it has no ownership).

class _EXPORT ScriptingView : public ScriptingElement
{
public:
	//	Flavourful constructors.

	ScriptingView(	const int32 theNumber = 0,
					BMessenger* theMessenger = 0,
					ScriptingElement* theParent = 0,
					const int32 theLevel = 0);
	ScriptingView(	const char* theName = 0,
					BMessenger* theMessenger = 0,
					ScriptingElement* theParent = 0,
					const int32 theLevel = 0);
	virtual	~ScriptingView();

	//	Display element on the console.
	
	virtual	void PrintToStream() const;

	//	Analyze view and its children.
	
	virtual	int32 Analyze(int32 theLevel = 0);
	
	//	Interface.
	#ifndef BESCRIPT
	virtual void UpdateItem();
	virtual const BBitmap* GetBitmap() const;
	#endif

private:
	void InitObject(const int32 theNumber);
	void InitObject(const char* theName);
	void InitObject();
};

//	FUNCTIONS:

_EXPORT	BList*					GetRunningApplications(const char* currentSignature);
_EXPORT	ScriptingApplication*	CreateScriptingApplication(const char* theAppIdentifier);

#endif	//	SCRIPTINGTARGETS_H
