/* Sample BeScript source. */
/* The language is case-sensitive. All command keywords are in lowercase only,
   but they appear in uppercase within comments for description purposes.
   Be-defined objects, such as Window, View, and MenuBar, are in MixedCase.
   UTF8 is supported within both comments and program text. */

/* Play with the Network preferences panel. That won't be used with BONE,
   so this example has limited life time… */

launch "application/x-vnd.Be-NETC"

/* TELL blocks use either an application name or MIME signature, whereas
   LAUNCH currently accepts MIME signatures only. */

tell Network
	/* Return number of open windows (usually 1). */
	count Window

	/* A Window specifier declared with "in" rather than "of" is dynamic.
	   BeScript will X-ray the remote window and attempt to resolve
	   the target. Moreover, unnamed views can be specified as well.
	   This is not possible with any other BeOS scripting language. */

	let View "Host name:" in Window "Network" do
		get Frame								/* Return dimensions. */
		get Value								/* Current host name. */
		set Value to "OurHostName"				/* Change it! */
	end

	/* Simple commands, using index (zero-based) and name specifiers. */

	get Minimize of Window 0
	set Title of Window "Network" to "WARNING! Host name has been changed!"
endtell

# Brutalize the BeIDE preferences and about box.
# A pound sign begins a comment until the end of the line, like // in C++.

launch "application/x-mw-BeIDE"
tell BeIDE
	/* Display the preferences window. Remeber that indices start at zero.
	/* The third window of the BeIDE is always a project or file window,
	   so it is guaranteed to have a MenuBar object.
	   We first display the window the hard way, pause for a while, then
	   bring it down and then up again -- this time, the easy way. :-)
	   We then click on an unnamed view using dynamic specifier resolution. */

	do MenuItem "Preferences…" of MenuBar of Window 2       # 2 == 3rd window
	set Title of Window "Environment Preferences" to
		"Environment Preferences (paused 2 secs)"

	print "Pausing two seconds..."
	pause 2000										# in milliseconds
	print " done!\n"

	set Title of Window "Environment Preferences (paused 2 secs)" to
		"Environment Preferences"
	quit Window "Environment Preferences"
	pause 1000

	# The easy way.
	# You can also use "in Window ...", but the specifier won't be dynamic
	# because dynamic specifiers don't make sense for menu items.

	choose "Preferences…" of Window 2					# 2 == 3rd window
	click button "Use an external editor" in Window "Environment Preferences"

	/* For some reason, this object does not support the system suite,
	   that is, it does not inherit from BListView or cancels support
	   for the suite. In any case, it's a pity because we can't touch it. */

	getsuites of View "listview" of Window "Environment Preferences"

	/* Test a generic verb. Note that this is equivalent to ABOUT.
	   Then, use a custom command defined by BWindow objects. */

	do '_ABR'
	let Window "About BeIDE" do
		do MoveBy BPoint(-250.0, 100.0)
		set Title to "About BeIDE (MOVED!)"
	end
endtell

/* Fireworks with the fonts demo application using dynamic specifiers. */

launch "application/x-vnd.Be-FDEM"
tell FontDemo

	# Begin a cycling display using our own settings for the demo.

	/* This is an example of implicit dynamic specifiers: the views
	   referenced by the following code are all expressed via their
	   screen names -- not their internal ones, which are unavailable or
	   useless. However, as you can see there are no explicit dynamic
	   specifiers here. Instead, BeScript will analyze the given window
	   when it encounters the first unknown specifier (View "Size:"),
	   and it will solve all specifiers in the block based on the analysis
	   performed at the LET level. The specifiers must still be resolved,
	   but at least the analysis is done only once… */

	let Window "Controls" do
		assign 64 to "Size:"
		assign 30 to "Rotation:"
		assign 120 to "Shear:"
		type "Sample Text" into "Text:"
		click "Cycle Fonts"
		pause 2500
		assign 60 to "Shear:"
		pause 2500
		assign 90 to "Shear:"
		assign 0 to "Rotation:"
		pause 5000
		click "RunThrough"
		/* This is the internal name of the button. At the moment we have
		   to use this bad trick because the dynamic analysis done by the
		   the LET block implicitly has recorded the *previous* name of
		   the button, i.e. "Cycle Fonts". I hope to solve this problem
		   in the future. */
	end
endtell
